/*
 * usb_audio.h
 *
 * An extension to the efm8_usb library.
 * This conforms to USB Audio Class 2.0.
 *
 *  Created on: Feb 1, 2019
 *      Author: andy
 */

#ifndef USB_AUDIO_H_
#define USB_AUDIO_H_

#include "si_toolchain.h"

#define USB_CLASS_AUDIO 0x01  				///< Audio device/interface class code.
#define USB_AUDIO_UNDEFINED 0x00 		///< Subclass undefined
#define USB_AUDIO_AUDIOCONTROL 0x01	///< Subclass Audio Control
#define USB_AUDIO_AUDIOSTREAMING 0x02 ///< Subclass Audio Streaming
#define USB_AUDIO_MIDISTREAMING 0x03	///< Subclass MIDI Streaming

#define USB_AUDIO_PROTOCOL_UNDEF 0x00 ///< Protocol undefined

#define USB_CS_AUDIO_UNDEFINED 0x20   ///< Class-specific descriptor undefined
#define USB_CS_AUDIO_DEVICE    0x21		///< Class-specific device descriptor
#define USB_CS_AUDIO_CONFIGURATION 0x022 ///< Class-specific configuration descriptor
#define USB_CS_AUDIO_STRING 0x23 ///< Class-specific string descriptor
#define USB_CS_AUDIO_INTERFACE 0x24 ///< Class-specific interface descriptor
#define USB_CS_AUDIO_ENDPOINT 0x25  ///< Class-specific endpoint descriptor

///< Class-specific Audio Control Interface Descriptor Subtypes
#define USB_CS_ACIF_UNDEFINED 0x00  ///< Undefined subtype
#define USB_CS_ACIF_HEADER 0x01
#define USB_CS_ACIF_INPUT_TERMINAL 0x02
#define USB_CS_ACIF_OUTPUT_TERMINAL 0x03
#define USB_CS_ACIF_MIXER_UNIT 0x04
#define USB_CS_ACIF_SELECTOR_UNIT 0x05
#define USB_CS_ACIF_FEATURE_UNIT 0x06
#define USB_CS_ACIF_EFFECT_UNIT 0x07
#define USB_CS_ACIF_PROCESSING_UNIT 0x08
#define USB_CS_ACIF_EXTENSION_UNIT 0x09
#define USB_CS_ACIF_CLOCK_SOURCE 0x0A
#define USB_CS_ACIF_CLOCK_SELECTOR 0x0B
#define USB_CS_ACIF_CLOCK_MULTIPLIER 0x0C
#define USB_CS_ACIF_SAMPLE_RATE_CONVERTER 0x0D

///< Class-specific Audio Streaming Interface Descriptor subtypes
#define USB_CS_ASIF_UNDEFINED 0x00
#define USB_CS_ASIF_GENERAL 0x01
#define USB_CS_ASIF_FORMAT_TYPE 0x02
#define USB_CS_ASIF_FORMAT_SPECIFIC 0x03

///< Processing Types.
#define USB_AUDIO_PROCESS_UNDEFINED 0x00
#define USB_AUDIO_UPDOWNMIX_PROCESS 0x01
#define USB_AUDIO_DOLBY_PROLOGIC_PROCESS 0x02
#define USB_AUDIO_3D_STEREO_EXTENDER_PROCESS 0x03
#define USB_AUDIO_REVERBERATION_PROCESS 0x04
#define USB_AUDIO_CHORUS_PROCESS 0x05
#define USB_AUDIO_DYN_RANGE_COMP_PROCESS 0x06

///< Audio Class-specific Endpoint Descriptor subtypes
#define USB_CS_EP_UNDEFINED 0x00
#define USB_CS_EP_GENERAL 0x01

///< Audio Class-specific request codes
#define USB_CS_REQ_UNDEFINED 0x00
#define USB_CS_REQ_SETCUR 0x01
#define USB_CS_REQ_GETCUR 0x81
#define USB_CS_REQ_SETMIN 0x02
#define USB_CS_REQ_GETMIN 0x82
#define USB_CS_REQ_SETMAX 0x03
#define USB_CS_REQ_GETMAX 0x83
#define USB_CS_REQ_SETRES 0x04
#define USB_CS_REQ_GETRES 0x84
#define USB_CS_REQ_SETMEM 0x05
#define USB_CS_REQ_GETMEM 0x85
#define USB_CS_REQ_GETSTAT 0xFF

///< Terminal Control Selectors
#define USB_AUDIO_TE_CONTROL_UNDEFINED 0x00
#define USB_AUDIO_COPY_PROTECT_CONTROL 0x01

/// @brief Feature Unit Control Selectors
#define USB_AUDIO_FU_CONTROL_UNDEFINED 0x00
#define USB_AUDIO_MUTE_CONTROL 0x01
#define USB_AUDIO_VOLUME_CONTROL 0x02
#define USB_AUDIO_BASS_CONTROL 0x03
#define USB_AUDIO_MID_CONTROL 0x04
#define USB_AUDIO_TREBLE_CONTROL 0x05
#define USB_AUDIO_GRAPHIC_EQUALIZER_CONTROL 0x06
#define USB_AUDIO_AUTOMATIC_GAIN_CONTROL 0x07
#define USB_AUDIO_DELAY_CONTROL 0x08
#define USB_AUDIO_BASS_BOOST_CONTROL 0x09
#define USB_AUDIO_LOUDNESS_CONTROL 0x0A

/// @brief Audio function category codes
#define USB_AUDIO_CATEGORY_UNDEFINED 0x0
#define USB_AUDIO_CATEGORY_DESKTOP_SPEAKER 0x1
#define USB_AUDIO_CATEGORY_HOME_THEATER 0x2
#define USB_AUDIO_CATEGORY_MICROPHONE 0x3
#define USB_AUDIO_CATEGORY_HEADSET 0x4
#define USB_AUDIO_CATEGORY_TELEPHONE 0x5
#define USB_AUDIO_CATEGORY_CONVERTER 0x6
#define USB_AUDIO_CATEGORY_VOICE_SOUND_RECORDER 0x7
#define USB_AUDIO_CATEGORY_IO_BOX 0x8
#define USB_AUDIO_CATEGORY_MUSICAL_INSTRUMENT 0x9
#define USB_AUDIO_CATEGORY_PROAUDIO 0xA
#define USB_AUDIO_CATEGORY_AUDIOVIDEO 0xB
#define USB_AUDIO_CATEGORY_CONTROL_PANEL 0xC
#define USB_AUDIO_CATEGORY_HEADPHONE 0xD
#define USB_AUDIO_CATEGORY_GENERIC_SPEAKER 0xE
#define USB_AUDIO_CATEGORY_HEADSET_ADAPTER 0xF
#define USB_AUDIO_CATEGORY_SPEAKERPHONE 0x10



 /// @brief USB class-specific Audio Control Interface Header Descriptor.
typedef struct
{
	uint8_t bLength;                   ///< Size of this descriptor in bytes.
	uint8_t bDescriptorType;           ///< Constant CS_INTERFACE Descriptor Type.
	uint8_t bDescriptorSubtype;        ///< Constant HEADER descriptor subtype.
	uint16_t bcdADC;					///< Audio Device Class Spec Release Number
	uint8_t bCategory;					///< Constant indicating primary use of this audio function
	uint16_t wTotalLength;				///< Total # of bytes returned for the CS AC IF descriptor
	uint8_t bmControls;					///< 1..0: latency, all else reserved
} USB_Audio_Control_Interface_Header_Descriptor_t;

/// @ clock source descriptor
typedef struct
{
	uint8_t bLength;	///< Size of this descriptor in bytes, 8
	uint8_t bDescriptorType; ///< CS_INTERFACE descriptor type
	uint8_t bDescriptorSubtype; ///< CLOCK_SOURCE descriptor subtype
	uint8_t bClockID;		///< identify the clock source in the function
	uint8_t bmAttributes;	///< clock type
	uint8_t bmControls;		///< frequency and validity control
	uint8_t bAssocTerminal;	///< Terminal ID of the terminal associated with this clock source
	uint8_t iClockSource;	///< Index of string descriptor for this clock source entity
} USB_Audio_Clock_Source_Descriptor_t;

/// @brief Constants for the two bitmaps above.
#define USB_AUDIO_CLOCK_ATTRIB_EXTERNAL 0x00	///< Clock is external
#define USB_AUDIO_CLOCK_ATTRIB_INT_FIXED 0x01	///< Clock is fixed-frequency internal
#define USB_AUDIO_CLOCK_ATTRIB_INT_VAR 0x02	///< Clock is variable-frequency internal
#define USB_AUDIO_CLOCK_ATTRIB_INT_PROG 0x03	///< Clock is programmable internal
#define USB_AUDIO_CLOCK_ATTRIB_SYNC_SOF 0x01	///< Clock is synchronized to USB SOF
















#endif /* USB_AUDIO_H_ */
