/*!
 * usb_midi.c
 * asp 19-04-30 asp.
 *
 * Handle MIDI messages received from the USB OUT Endpoint.
 *
 * We set the Endpoint's max packet size to 327. This ensures that for any packet
 * sent from the host we will have either one or two 4-byte MIDI messages. When
 * the endpoint interrupt handler is called, it will read the messages from the
 * endpoint FIFO and push them to the MIDI Message FIFO. The main loop will
 * check to see if there are any messages, and pop them one at a time for parsing.
 *
 * The MIDI Message FIFO holds four messages (32 byte). I will at some point
 * create a generic FIFO type which holds the head/tail/count as well as the
 * buffer whose size can be anything. For now it's created here.
 *
 * MODS.
 * 2019-05-07 Stall endpoint when our MIDI message buffer is almost full, and
 * 	unstall it when we've popped enough to allow more.
 * 2019-05-21 We can't stall endpoints, that doesn't work. Something else has to
 * 	manage holding off the OUT transactions, but not here.
 * 2019-05-21 Refactor to make clear that the functions deal with the USB MIDI
 *  data FIFO.
 * 2019-06-10.
 */

#include <si_toolchain.h>
#include <SI_EFM8UB2_Register_Enums.h>                  // SFR declarations
#include <efm8_usb.h>
#include "usb_midi.h"

// How many messages will fit into our FIFO?
#ifndef MIDI_USB_FIFO_SIZE
#define MIDI_USB_FIFO_SIZE 32
#endif

/*
 * Define a software FIFO for the MIDI messages. We need pointers and a buffer.
 * pointers are in DATA space for speed. Buffer is in XDATA so we have room for it.
 */
typedef struct {
	uint8_t head;						/*!< Index of the head of the FIFO */
	uint8_t tail;						/*!< Index of the tail of the FIFO */
	uint8_t count;						/*!< Number of messages in the FIFO */
	USBMIDI_Message_t msg[MIDI_USB_FIFO_SIZE]; /*!< the buffer */
} USBMIDIFIFO_t;

static xdata USBMIDIFIFO_t midififo;

/*
 * Initialize the MIDI message FIFO.
 * Clear the contents to make debug easier.
 */
void USBMIDIFIFO_Init(void)
{
	uint8_t i;

	for (i= 0; i < MIDI_USB_FIFO_SIZE; i++) {
		midififo.msg[i].header = 0x00;
		midififo.msg[i].byte1 = 0x00;
		midififo.msg[i].byte2 = 0x00;
		midififo.msg[i].byte3 = 0x00;
	}
	midififo.head = 0;
	midififo.tail = 0;
	midififo.count = 0;
} // MIDIFIFO_Init()

/**
 * Push a new message onto the FIFO.
 * @bug Do what in case of overflow?
 * @param msg The MIDI message to push onto the FIFO.
 */
void USBMIDIFIFO_Push(USBMIDI_Message_t xdata *msg)
{
	// DEBUG TEST POINT
	if (midififo.count >= MIDI_USB_FIFO_SIZE)
		P3_B0 = 1;
	else
		P3_B0 = 0;

	midififo.msg[midififo.head] = *msg;
	midififo.head++;
	if (midififo.head >= MIDI_USB_FIFO_SIZE) {
		midififo.head = 0;
	}
	midififo.count++;
} // MIDIFIFO_Push()

/**
 * Pop a message from the USB MIDI FIFO.
 * This fetches a message that was sent from the host on an OUT endpoint.
 * @returns true if we actually popped something, else false if the FIFO was
 * empty.
 * @param msg: this is the message popped from the FIFO.
 */
bool USBMIDIFIFO_Pop(USBMIDI_Message_t xdata *msg)
{
	if (midififo.count == 0)
		// nothing to pop, so caller doesn't parse any message.
		return false;

	*msg = midififo.msg[midififo.tail];
	midififo.tail++;
	if (midififo.tail >= MIDI_USB_FIFO_SIZE) {
		midififo.tail = 0;
	}
	midififo.count--;

	return true;
} // MIDIFIFO_Pop()
